# Transmission CLI
# source    https://github.com/gotbletu/dotfiles
#           https://wiki.archlinux.org/index.php/transmission#Reducing_journal_spam
# blocklist https://github.com/gotbletu/dotfiles/blob/master/scripts/.scripts/blocklist.sh
# WEBUI:    http://localhost:9091/transmission/web/

ts-clear() {
  transmission-remote -l | grep 100% | grep Done | \
  awk '{print $1}' | xargs -n 1 -I % transmission-remote -t % -r
}

# display numbers of ip being blocked by the blocklist (credit: smw from irc #transmission)
ts-count() {
  echo "Blocklist rules:" $(curl -s --data \
  '{"method": "session-get"}' localhost:9091/transmission/rpc -H \
  "$(curl -s -D - localhost:9091/transmission/rpc | grep X-Transmission-Session-Id)" \
  | cut -d: -f 11 | cut -d, -f1)
}

ts-d() { transmission-daemon --foreground --log-info 2>&1 | while read line; do
    echo $line |
        grep -v "announcer.c:\|platform.c:\|announce done (tr-dht.c:" |
        grep -v "Saved.*variant.c:" |
        while read line; do
            echo $line | grep -q "Queued for verification (verify.c:" &&
                notify-send --app-name="Transmission Started" "${line#* * }"
            echo $line | grep -q "changed from .Incomplete. to .Complete." &&
                notify-send --app-name="Transmission Complete" "${line#* * }"
            echo $line | systemd-cat --identifier="TransWrap" --priority=5
        done 2>&1 > /dev/null
    done&disown ;}

ts-block() {
  echo "${Red} Stopping Transmission Daemon ${Color_Off}"
    ts-q
  echo "${Yellow} Updating Blocklist ${Color_Off}"
    ~/bin/blocklist.sh
  echo "${Green} Restarting Transmission Daemon ${Color_Off}"
    ts-d
    sleep 3
  echo "${Blue}╓───────────── ─ ─ Number of IPs Blocked ─ ─ ─────────────╖${Color_Off}"
    ts-count
  echo "╙─────────────────────────────────────────────────────╜"  
}

ts-add() {
    TRANS_PID=$(pidof transmission-daemon)

    if [[ $TRANS_PID ]]; then
        transmission-remote --add "$1"
    else
        ts-d
        sleep 5
        transmission-remote --add "$1"
    fi
;}

ts-q() { killall transmission-daemon 2> /dev/null ;}
ts-peers() { transmission-remote -t"$1" --reannounce ;}
ts-pause() { transmission-remote -t"$1" --stop ;}              	# <id> or all
ts-go() { transmission-remote -t"$1" --start ;}             	# <id> or all
ts-del() { transmission-remote -t"$1" --remove-and-delete ;} 	# delete data also
ts-rm() { transmission-remote -t"$1" --remove ;}           		# leaves data alone
ts-info() { transmission-remote -t"$1" --info ;}
ts-ls() { transmission-remote --list ;}